#!/usr/bin/perl
#
# Shutdownscript for extb-Board
#
# adapt by Andreas Brachold <vdr04-at-deltab.de>
# based on Shutdownscript for ACPI by Thomas Koch <tom-at-linvdr.org>
#
use strict;
use POSIX qw(strftime sprintf localtime time);
use Time::Local;
################################################################################
# How many Seconds should poweron before timer started
my $STARTUPMARGIN = 300;
# How many second need your system for Shutdown, extb-timer run not until power is down
my $SHUTDOWNDURATION = 15; 	
# How many seconds can drift the timer on one day
my $DAILYOFFSET = 0;
# Wake avery night at 02:00 for e.g. EPG Scan
my $WAKEFOREPGSCAN = 0;
# Dump any messages on Screen and don't go powerdown
my $DEBUG = 0;

################################################################################
# Internal Limits and change only if you know what you do
# How long should extb sleep, eg only 1 Day, 1 Week , 1 Month ... if used timer delay bigger
my $SLEEPLIMIT = 0xFFFFFF; # FFFFFF are 2^24 * 0.524288 Sek = 8.796.093 Sek = 146.601 Min = 101,8 Days
# Duration of one counter step inside the PIC,  4MHz are 0.524288 sek
my $PICTICKER = 0.524288;  
# Which command should call to transfer IR Codes to extb-board rc or irsend
my $RCCMD = "/usr/bin/irsend";
################################################################################

sub dprint {
  $_ = join("", @_);
  chomp;
  print "$_\n" if($DEBUG);
}

sub setTime {
  system("hwclock -w"); 
}  

sub setAlarm {
  my $Next = shift;
  setTime();  
  dprint("Next event at ", strftime("%d.%m.%Y %H:%M:%S", localtime($Next)));
  dprint("Programming wakeup at ", strftime("%d.%m.%Y %H:%M:%S", localtime($Next-$STARTUPMARGIN)));

  # Get difference between now and next timer
  my $delta = (($Next - ($STARTUPMARGIN + $SHUTDOWNDURATION)) - time());
  if($delta < $STARTUPMARGIN)
  {
    dprint("Wakeup to short ",$delta, " Seconds");
    exit 1;
  }

  dprint("Wakeup at ",int($delta), " Seconds");
  # Adjust drift crystal with 6 Sec on one day
  if( $DAILYOFFSET > 0)    
  {
    $delta -= ($delta / 86400)*$DAILYOFFSET; # 24h*60m*60s
    dprint("Adjusted Wakeup at ",int($delta), " Seconds");
  }


  # Adjust for PIC Counterticks
  my $valueint =  $delta / $PICTICKER;
  dprint("PIC value ", int($valueint), " PicTics");

  # Userclipping
  if(int($valueint) > $SLEEPLIMIT){ $valueint = $SLEEPLIMIT;  }
  # Systemclipping
  if(int($valueint) > 0xFFFFFF)   { $valueint = 0xFFFFFF;     }

  # integer to hex ( 6 Characters)
  my $valuehex = uc(sprintf("%06x",int($valueint)));
  # insert between all hex character a placeholder "space"
  my $valuestring = "";
  for(my $i = 0; $i <= length($valuehex); $i+=1){
  $valuestring = $valuestring.substr($valuehex,$i,1)." ";
  };
  # Dump only command on screen if use Debug
  system( sprintf("echo %s send_once EXTB_TX %s",$RCCMD,$valuestring )) if($DEBUG);
  # Execute Commandoline if'nt use Debug
  system( sprintf("%s send_once EXTB_TX %s",$RCCMD,$valuestring )) if(!$DEBUG);
}

sub VideoOff {
  # Turn Video off, if'nt use Debug
  system( sprintf("echo %s send_once EXTB_TX %s",$RCCMD,"M2")) if($DEBUG);
  system( sprintf("%s send_once EXTB_TX %s",$RCCMD,"M2")) if(!$DEBUG);
}

sub PowerOff {
  # Busybox shutdown
 system( sprintf("echo /bin/busybox poweroff")) if($DEBUG);
 system("/bin/busybox poweroff") if(!$DEBUG);
  # Real System shutdown if'nt use Debug
#  system("/sbin/shutdown -h now") if(!$DEBUG);
}


sub NextTwoOclock {
  my $now = time();
  # Array-Format: ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
  my @today = localtime($now);
  @today[0..2] = (0, 0, 2);

  # get today 2:00 in seconds since epoc
  my $TwoOclock = timelocal(@today);
  
  # Check: Is today 2:00 in future or past?
  if($now < $TwoOclock) {
    # Today two oclock is in future
    return $TwoOclock;
  } else {
    # We're past two oclock, next two oclock is tomorrow (+86400s)
    return $TwoOclock+86400;
  }
}

my $TwoOclock = NextTwoOclock();

if(scalar(@ARGV)) {
  # called from vdr
  die "Wrong parameter count\n" if(scalar(@ARGV) != 5);

  my($Next, $Delta, $Channel, $Recording, $UserShutdown) = @ARGV;
  # dprint ("$Next, $Delta, $Channel, $Recording, $UserShutdown");
  VideoOff();
  # find out: Next start at 2:00 or at next timer?
  if(($Next) && (!($WAKEFOREPGSCAN) || ($Next < $TwoOclock))) {
    setAlarm($Next);
  } else {
    if($WAKEFOREPGSCAN) {
        setAlarm($TwoOclock);
    }
  }
  PowerOff();
} else {
  # called from cmdline
  my $next = `svdrpsend.pl next abs`;
  if($next =~ /550 No active timers/) {
    VideoOff();
    if($WAKEFOREPGSCAN) {
    # start every day at 2:00 local time for EPG update & Co.
    setAlarm($TwoOclock);
    }
    PowerOff();
  } elsif($next =~ /250 \d+ (\d+)/) {
    VideoOff();
    # find out: Next start at 2:00 or at next timer?
    if(($1 < $TwoOclock) || !($WAKEFOREPGSCAN)) {
      setAlarm($1);
    } else {
        if($WAKEFOREPGSCAN) {
            setAlarm($TwoOclock);
        }
    }
    PowerOff();
  } else {
    print "HELP! What to do? ($next)\n";
  }
}	
